/*
 radmonfix

   Authors:  Aitor Ibarra - XMM-Newton SOC

   Description: Add TIME column to radmon files

   Comments:

  $Id: radmonfix.cc,v 1.3 2010/04/01 11:54:28 aibarra Exp $
*/

#include "radmonfix.h"

#include <Tracer.h>
#include <errstr.h>
#include "TimeTags.h"
#include "OalServer.h"
#include <SasDalUtils.h>

void radmonfix ()
{
PROC

	//const Odf *odf = oalServer.newOdf("./");
	//errstr<<*odf<<Sas::message(Msg::App, Msg::Verbose);

	//
	//	read the frame numbers in the dummy EMOS1 auxiliary file
	//
	//const OdfSet *aux = oalServer.odf()->selectSet(Xmm::ERM, Xmm::AUXILIARY);
	//assert(aux);

	//cout << "Rev number " << oalServer.odf()->orbitNr() << endl;
	oalServer.instrument(Xmm::ERM);

	// Open input event list and source list files
	vector<string> radmonVec = stringListParameter("radmonlist");

	for (unsigned int i = 0 ; i < radmonVec.size() ; ++i)
	{
		DataSet *set = dataSetServer->open(radmonVec[i], DataSet::Modify);
		Table *tab = set->table(0);

		ColumnData *cdframes = tab->column("FRAME")->data();
		int32 *frames = cdframes->int32Data();

		const FrameCounterVector *frameVec = oalServer.newFrameCounterVector(frames,tab->rows());

		frameVec->setAux(radmonVec[i]);
		ObtVector obtvec(frameVec->toObt());

		vector<double> utcVec(obtvec.toMET());
		if (utcVec.size() != tab->rows())
		{
			errstr << "Time convertion error. Incorrect number of counts "
			<< Sas::error("IncorrectNumberOfRows");
			//assert(utcVec.size() == tab->rows());
		}


		/*
		 * Adding UTC time in secs
		 */
		Column * timeCol = 0;
		if (tab->hasColumn("TIME"))
		{
			errstr << "TIME Column already exits. Overwriting TIME Column..."
				<< Sas::message(Msg::App, Msg::Sparse);
			timeCol = tab->column("TIME",DataSet::Modify);
		} else
		{
			errstr << "TIME Column does not exit. Creating TIME Column..."
				<< Sas::message(Msg::App, Msg::Sparse);

			timeCol =tab->addColumn("TIME",
				     Column::Real64,
				     "Time",
				     "secs");
		}
		ColumnData * timeColData = timeCol->data();
		real64 * timeData = timeColData->real64Data();

		/*
		 * Adding Modify Julian Date
		 */

		Column * mjdCol = 0;
		if (tab->hasColumn("MJD"))
		{
			errstr << "MJD Column already exits. Overwriting MJD Column..."
				<< Sas::message(Msg::App, Msg::Sparse);
			mjdCol = tab->column("MJD",DataSet::Modify);
		} else
		{
			errstr << "TIME Column does not exit. Creating TIME Column..."
				<< Sas::message(Msg::App, Msg::Sparse);

			mjdCol =tab->addColumn("MJD",
				     Column::Real64,
				     "Modified Julian Date",
				     "secs");
		}


		ColumnData * mjdColData = mjdCol->data();
		real64 * mjdData = mjdColData->real64Data();


		/*
		 * Adding UTC time (Calendar view)
		 */

		Column * calCol = 0;
		longVec lv(1); lv[0] = 21;

		if (tab->hasColumn("TIMECAL"))
		{
			errstr << "UTC Calendar view Column already exits. Overwriting UTC Calendar Column..."
				<< Sas::message(Msg::App, Msg::Sparse);
			calCol = tab->column("TIMECAL",DataSet::Modify);
		} else
		{
			errstr << "UTC Calendar Column does not exit. Creating UTC Calendar Column..."
				<< Sas::message(Msg::App, Msg::Sparse);
			calCol =tab->addColumn("TIMECAL",
					 Dal::DString,
				     "Time",
				     "yyyy:mm:ddThh:mm:ss.s",lv);
		}

		ColumnData * calColData = calCol->data();


		//cout<<"Frame counters - OBT - tag - time:\n";
		cout.precision(16);
		for (unsigned n=0; n<utcVec.size(); ++n) {
			STime t = Xmm::missionReferenceTime + utcVec[n];
			//cout<<frames[n]<<" "<<obtvec[n]<<" "<<utcVec[n]<<" "
			//	<<t.toString()<< "  " << t.modifiedJulianDay().full()<< endl;
			timeData[n] = utcVec[n];
			mjdData[n] = t.modifiedJulianDay().full();
			SasDalUtils::setStringColumnRow(calColData,n, t.toString(1, STime::FITS));



		}

		delete calColData;
		delete mjdColData;
		delete timeColData;
		delete cdframes;

		//Attributes
		*tab->addAttribute("TASSIGN", "Location of time assignment") = "SATELLITE";
		*tab->addAttribute("TIMEREF", "Reference location of photon arrival times") = "LOCAL";
		*tab->addAttribute("TIMESYS", "XMM time will be TT (Terrestial Time)") = "TT";
		*tab->addAttribute("MJDREF", "1998-01-01T00:00:00 (TT) expressed in MJD (TT)","d") = "50814D+0";


		dataSetServer->close(set);

	}

ENDPROC
}
